{% load static %}
<!DOCTYPE html>
<html>
<head>
    <link rel="shortcut icon" href="{% static 'ninja/favicon.png' %}">
    <title>{{ api.title }}</title>
</head>
<body>
    <redoc spec-url='{{ openapi_json_url }}'></redoc>
    <script src="{% static 'ninja/redoc.standalone.js' %}"></script>
</body>
</html>
