<!DOCTYPE html>
<html>
<head>
    <link rel="shortcut icon" href="https://django-ninja.rest-framework.com/img/favicon.png">
    <title>{{ api.title }}</title>
</head>
<body>
    <redoc spec-url='{{ openapi_json_url }}'></redoc>
    <script src="https://cdn.jsdelivr.net/npm/redoc@2.0.0/bundles/redoc.standalone.js"></script>
</body>
</html>
